@@ -21,7 +21,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef linux
-#include <asm/page.h>
 #include <linux/elf.h>
 #define ELFDATA2MSB   2
 #else
@@ -409,7 +408,7 @@ void print_ptab64(ProgTable64 *t) {
 }
 
 void load_image(Segment *t, const ProgTable *tp, FILE *inp) {
-	const Elf32_Phdr *p;
+	Elf32_Phdr *p, *q;
 	unsigned x;
 	unsigned long off, len;
 
@@ -417,10 +416,20 @@ void load_image(Segment *t, const ProgTable *tp, FILE *inp) {
 	for (x = 0; x < tp->len; x++) {
 		if (tp->tab[x].p_type == PT_LOAD) {
 			if (p != 0) {
-				fprintf(stderr, "Several loadable segments\n");
-				exit(1);
-			}
-			p = &tp->tab[x];
+				q = &tp->tab[x];
+				off = (q->p_offset - p->p_offset);
+				if (q->p_vaddr - p->p_vaddr == off &&
+				    q->p_paddr - p->p_paddr == off &&
+				    p->p_memsz == p->p_filesz &&
+				    p->p_memsz <= off) {
+					p->p_filesz = off + q->p_filesz;
+					p->p_memsz = off + q->p_memsz;
+				} else {
+					fprintf(stderr, "Several loadable segments\n");
+					exit(1);
+				}
+			} else
+				p = &tp->tab[x];
 		}
 	}
 	if (p == 0) {
@@ -467,7 +476,7 @@ void load_image(Segment *t, const ProgTable *tp, FILE *inp) {
 }
 
 void load_image64(Segment *t, const ProgTable64 *tp, FILE *inp) {
-	const Elf64_Phdr *p;
+	Elf64_Phdr *p, *q;
 	unsigned x;
 	unsigned long long off, len;
 
@@ -475,10 +484,20 @@ void load_image64(Segment *t, const ProgTable64 *tp, FILE *inp) {
 	for (x = 0; x < tp->len; x++) {
 		if (tp->tab[x].p_type == PT_LOAD) {
 			if (p != 0) {
-				fprintf(stderr, "Several loadable segments\n");
-				exit(1);
-			}
-			p = &tp->tab[x];
+				q = &tp->tab[x];
+				off = (q->p_offset - p->p_offset);
+				if (q->p_vaddr - p->p_vaddr == off &&
+				    q->p_paddr - p->p_paddr == off &&
+				    p->p_memsz == p->p_filesz &&
+				    p->p_memsz <= off) {
+					p->p_filesz = off + q->p_filesz;
+					p->p_memsz = off + q->p_memsz;
+				} else {
+					fprintf(stderr, "Several loadable segments\n");
+					exit(1);
+				}
+			} else
+				p = &tp->tab[x];
 		}
 	}
 	if (p == 0) {
@@ -562,6 +581,6 @@ void Usage(){
 }
 
 void Version(){
-	printf("elftoaout 2.1: ELF to a.out convertor for SPARC and SPARC64 bootstraps\n");
+	printf("elftoaout 2.2: ELF to a.out convertor for SPARC and SPARC64 bootstraps\n");
 }
 
